@@ -78,6 +78,8 @@ struct _ECalBackendFilePrivate {
 
 
 
+#define d(x)
+
 static void e_cal_backend_file_dispose (GObject *object);
 static void e_cal_backend_file_finalize (GObject *object);
 
@@ -359,7 +361,7 @@ check_dup_uid (ECalBackendFile *cbfile, ECalComponent *comp)
 	if (!obj_data)
 		return; /* Everything is fine */
 
-	g_message ("check_dup_uid(): Got object with duplicated UID `%s', changing it...", uid);
+	d(g_message (G_STRLOC ": Got object with duplicated UID `%s', changing it...", uid));
 
 	new_uid = e_cal_component_gen_uid ();
 	e_cal_component_set_uid (comp, new_uid);
@@ -401,38 +403,46 @@ add_component (ECalBackendFile *cbfile, ECalComponent *comp, gboolean add_to_top
 
 	priv = cbfile->priv;
 
-	if (e_cal_component_is_instance (comp)) { /* FIXME: more checks needed, to detect detached instances */
+	e_cal_component_get_uid (comp, &uid);
+	obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
+	if (e_cal_component_is_instance (comp)) {
 		const char *rid;
-
-		e_cal_component_get_uid (comp, &uid);
-
-		obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
-		if (!obj_data) {
-			g_warning (G_STRLOC ": Got an instance of a non-existing component");
-			return;
-		}
-
+	
 		rid = e_cal_component_get_recurid_as_string (comp);
-		if (g_hash_table_lookup (obj_data->recurrences, rid)) {
-			g_warning (G_STRLOC ": Tried to adding an already existing recurrence");
-			return;
+		if (obj_data) {
+			if (g_hash_table_lookup (obj_data->recurrences, rid)) {
+				g_warning (G_STRLOC ": Tried to add an already existing recurrence");
+				return;
+			}
+		} else {
+			obj_data = g_new0 (ECalBackendFileObject, 1);
+			obj_data->full_object = NULL;
+			obj_data->recurrences = g_hash_table_new (g_str_hash, g_str_equal);
+			g_hash_table_insert (priv->comp_uid_hash, (gpointer) uid, obj_data);
 		}
 
 		g_hash_table_insert (obj_data->recurrences, g_strdup (rid), comp);
-		/* FIXME: sort the recurrences */
 		obj_data->recurrences_list = g_list_append (obj_data->recurrences_list, comp);
 	} else {
 		/* Ensure that the UID is unique; some broken implementations spit
 		 * components with duplicated UIDs.
 		 */
 		check_dup_uid (cbfile, comp);
-		e_cal_component_get_uid (comp, &uid);
 
-		obj_data = g_new0 (ECalBackendFileObject, 1);
-		obj_data->full_object = comp;
-		obj_data->recurrences = g_hash_table_new (g_str_hash, g_str_equal);
+		if (obj_data) {
+			if (obj_data->full_object) {
+				g_warning (G_STRLOC ": Tried to add an already existing object");
+				return;
+			}
 
-		g_hash_table_insert (priv->comp_uid_hash, (gpointer) uid, obj_data);
+			obj_data->full_object = comp;
+		} else {
+			obj_data = g_new0 (ECalBackendFileObject, 1);
+			obj_data->full_object = comp;
+			obj_data->recurrences = g_hash_table_new (g_str_hash, g_str_equal);
+
+			g_hash_table_insert (priv->comp_uid_hash, (gpointer) uid, obj_data);
+		}
 	}
 
 	priv->comp = g_list_prepend (priv->comp, comp);
@@ -1162,16 +1172,18 @@ match_object_sexp (gpointer key, gpointer value, gpointer data)
 	ECalBackendFileObject *obj_data = value;
 	MatchObjectData *match_data = data;
 
-	if ((!match_data->search_needed) ||
-	    (e_cal_backend_sexp_match_comp (match_data->obj_sexp, obj_data->full_object, match_data->backend))) {
-		match_data->obj_list = g_list_append (match_data->obj_list,
-						      e_cal_component_get_as_string (obj_data->full_object));
-
-		/* match also recurrences */
-		g_hash_table_foreach (obj_data->recurrences,
-				      (GHFunc) match_recurrence_sexp,
-				      match_data);
+	if (obj_data->full_object) {
+		if ((!match_data->search_needed) ||
+		    (e_cal_backend_sexp_match_comp (match_data->obj_sexp, obj_data->full_object, match_data->backend))) {
+			match_data->obj_list = g_list_append (match_data->obj_list,
+							      e_cal_component_get_as_string (obj_data->full_object));
+		}
 	}
+
+	/* match also recurrences */
+	g_hash_table_foreach (obj_data->recurrences,
+			      (GHFunc) match_recurrence_sexp,
+			      match_data);
 }
 
 /* Get_objects_in_range handler for the file backend */
@@ -1185,7 +1197,7 @@ e_cal_backend_file_get_object_list (ECalBackendSync *backend, EDataCal *cal, con
 	cbfile = E_CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
 
-	g_message (G_STRLOC ": Getting object list (%s)", sexp);
+	d(g_message (G_STRLOC ": Getting object list (%s)", sexp));
 
 	match_data.search_needed = TRUE;
 	match_data.query = sexp;
@@ -1218,7 +1230,7 @@ e_cal_backend_file_start_query (ECalBackend *backend, EDataCalView *query)
 	cbfile = E_CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
 
-	g_message (G_STRLOC ": Starting query (%s)", e_data_cal_view_get_text (query));
+	d(g_message (G_STRLOC ": Starting query (%s)", e_data_cal_view_get_text (query)));
 
 	/* try to match all currently existing objects */
 	match_data.search_needed = TRUE;
@@ -1413,7 +1425,7 @@ typedef struct
 } ECalBackendFileComputeChangesData;
 
 static void
-e_cal_backend_file_compute_changes_foreach_key (const char *key, gpointer data)
+e_cal_backend_file_compute_changes_foreach_key (const char *key, gpointer value, gpointer data)
 {
 	ECalBackendFileComputeChangesData *be_data = data;
 	
@@ -1435,7 +1447,7 @@ e_cal_backend_file_compute_changes_foreach_key (const char *key, gpointer data)
 
 static ECalBackendSyncStatus
 e_cal_backend_file_compute_changes (ECalBackendFile *cbfile, const char *change_id,
-				  GList **adds, GList **modifies, GList **deletes)
+				    GList **adds, GList **modifies, GList **deletes)
 {
 	ECalBackendFilePrivate *priv;
 	char    *filename;
@@ -1449,9 +1461,13 @@ e_cal_backend_file_compute_changes (ECalBackendFile *cbfile, const char *change_
 	/* FIXME Will this always work? */
 	unescaped_uri = gnome_vfs_unescape_string (priv->uri, "");
 	filename = g_strdup_printf ("%s-%s.db", unescaped_uri, change_id);
-	ehash = e_xmlhash_new (filename);
-	g_free (filename);
 	g_free (unescaped_uri);
+	if (!(ehash = e_xmlhash_new (filename))) {
+		g_free (filename);
+		return GNOME_Evolution_Calendar_OtherError;
+	}
+	
+	g_free (filename);
 	
 	/* Calculate adds and modifies */
 	for (i = priv->comp; i != NULL; i = i->next) {
@@ -1485,8 +1501,9 @@ e_cal_backend_file_compute_changes (ECalBackendFile *cbfile, const char *change_
 	be_data.kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbfile));
 	be_data.deletes = NULL;
 	be_data.ehash = ehash;
-   	e_xmlhash_foreach_key (ehash, (EXmlHashFunc)e_cal_backend_file_compute_changes_foreach_key, &be_data);
-
+	
+	e_xmlhash_foreach_key (ehash, (EXmlHashFunc)e_cal_backend_file_compute_changes_foreach_key, &be_data);
+	
 	*deletes = be_data.deletes;
 
 	e_xmlhash_write (ehash);
@@ -1607,7 +1624,7 @@ sanitize_component (ECalBackendFile *cbfile, ECalComponent *comp)
 
 
 static ECalBackendSyncStatus
-e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, const char *calobj, char **uid)
+e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, char **calobj, char **uid)
 {
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
@@ -1620,10 +1637,10 @@ e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, const
 	priv = cbfile->priv;
 
 	g_return_val_if_fail (priv->icalcomp != NULL, GNOME_Evolution_Calendar_NoSuchCal);
-	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_ObjectNotFound);
+	g_return_val_if_fail (*calobj != NULL, GNOME_Evolution_Calendar_ObjectNotFound);
 
 	/* Parse the icalendar text */
-	icalcomp = icalparser_parse_string ((char *) calobj);
+	icalcomp = icalparser_parse_string (*calobj);
 	if (!icalcomp)
 		return GNOME_Evolution_Calendar_InvalidObject;
 
@@ -1635,7 +1652,21 @@ e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, const
 
 	/* Get the UID */
 	comp_uid = icalcomponent_get_uid (icalcomp);
-	
+	if (!comp_uid) {
+		char *new_uid;
+
+		new_uid = e_cal_component_gen_uid ();
+		if (!new_uid) {
+			icalcomponent_free (icalcomp);
+			return GNOME_Evolution_Calendar_InvalidObject;
+		}
+
+		icalcomponent_set_uid (icalcomp, new_uid);
+		comp_uid = icalcomponent_get_uid (icalcomp);
+
+		g_free (new_uid);
+	}
+
 	/* check the object is not in our cache */
 	if (lookup_component (cbfile, comp_uid)) {
 		icalcomponent_free (icalcomp);
@@ -1660,9 +1691,10 @@ e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, const
 	/* Save the file */
 	save (cbfile);
 
-	/* Return the UID */
+	/* Return the UID and the modified component */
 	if (uid)
 		*uid = g_strdup (comp_uid);
+	*calobj = e_cal_component_get_as_string (comp);
 
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -1722,12 +1754,29 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	case CALOBJ_MOD_THIS :
 		rid = e_cal_component_get_recurid_as_string (comp);
 		if (!rid || !*rid) {
-			g_object_unref (comp);
-			return GNOME_Evolution_Calendar_ObjectNotFound;
+			if (old_object)
+				*old_object = e_cal_component_get_as_string (obj_data->full_object);
+
+			/* replace only the full object */
+			icalcomponent_remove_component (priv->icalcomp,
+							e_cal_component_get_icalcomponent (obj_data->full_object));
+			priv->comp = g_list_remove (priv->comp, obj_data->full_object);
+
+			/* add the new object */
+			g_object_unref (obj_data->full_object);
+			obj_data->full_object = comp;
+
+			icalcomponent_add_component (priv->icalcomp,
+						     e_cal_component_get_icalcomponent (obj_data->full_object));
+			priv->comp = g_list_prepend (priv->comp, obj_data->full_object);
+
+			save (cbfile);
+
+			return GNOME_Evolution_Calendar_Success;
 		}
 
 		if (g_hash_table_lookup_extended (obj_data->recurrences, rid,
-						  &real_rid, &recurrence)) {
+						  (void **) &real_rid, (void **) &recurrence)) {
 			if (old_object)
 				*old_object = e_cal_component_get_as_string (recurrence);
 
@@ -1800,7 +1849,7 @@ remove_instance (ECalBackendFile *cbfile, ECalBackendFileObject *obj_data, const
 	if (!rid || !*rid)
 		return;
 
-	if (g_hash_table_lookup_extended (obj_data->recurrences, rid, &hash_rid, &comp)) {
+	if (g_hash_table_lookup_extended (obj_data->recurrences, rid, (void **) &hash_rid, (void **) &comp)) {
 		/* remove the component from our data */
 		icalcomponent_remove_component (cbfile->priv->icalcomp,
 						e_cal_component_get_icalcomponent (comp));
@@ -1831,6 +1880,8 @@ remove_instance (ECalBackendFile *cbfile, ECalBackendFileObject *obj_data, const
 	/* add the modified object to the beginning of the list, 
 	   so that it's always before any detached instance we
 	   might have */
+	icalcomponent_add_component (cbfile->priv->icalcomp,
+				     e_cal_component_get_icalcomponent (obj_data->full_object));
 	cbfile->priv->comp = g_list_prepend (cbfile->priv->comp, obj_data->full_object);
 }
 
@@ -1882,8 +1933,8 @@ remove_object_instance_cb (gpointer key, gpointer value, gpointer user_data)
 /* Remove_object handler for the file backend */
 static ECalBackendSyncStatus
 e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
-				const char *uid, const char *rid,
-				CalObjModType mod, char **object)
+				  const char *uid, const char *rid,
+				  CalObjModType mod, char **object)
 {
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
@@ -1910,16 +1961,19 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 		remove_component (cbfile, comp);
 		break;
 	case CALOBJ_MOD_THIS :
+		*object = e_cal_component_get_as_string (comp);
 		if (!rid || !*rid)
-			return GNOME_Evolution_Calendar_ObjectNotFound;
-
-		remove_instance (cbfile, obj_data, rid);
+			remove_component (cbfile, comp);
+		else
+			remove_instance (cbfile, obj_data, rid);
 		break;
 	case CALOBJ_MOD_THISANDPRIOR :
 	case CALOBJ_MOD_THISANDFUTURE :
 		if (!rid || !*rid)
 			return GNOME_Evolution_Calendar_ObjectNotFound;
 
+		*object = e_cal_component_get_as_string (comp);
+
 		/* remove the component from our data, temporarily */
 		icalcomponent_remove_component (priv->icalcomp,
 						e_cal_component_get_icalcomponent (comp));
@@ -1990,7 +2044,7 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 	icalcomponent_kind kind;
 	icalproperty_method method;
 	icalcomponent *subcomp;
-	GList *comps, *l;
+	GList *comps, *del_comps, *l;
 	ECalComponent *comp;
 	struct icaltimetype current;
 	ECalBackendFileTzidData tzdata;
@@ -2032,44 +2086,44 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 	}	
 
 	/* First we make sure all the components are usuable */
-	comps = NULL;
+	comps = del_comps = NULL;
+	kind = e_cal_backend_get_kind (E_CAL_BACKEND (backend));
+
 	subcomp = icalcomponent_get_first_component (toplevel_comp, ICAL_ANY_COMPONENT);
 	while (subcomp) {
-		/* We ignore anything except VEVENT, VTODO and VJOURNAL
-		   components. */
 		icalcomponent_kind child_kind = icalcomponent_isa (subcomp);
-
-		switch (child_kind) {
-		case ICAL_VEVENT_COMPONENT:
-		case ICAL_VTODO_COMPONENT:
-		case ICAL_VJOURNAL_COMPONENT:
-			tzdata.found = TRUE;
-			icalcomponent_foreach_tzid (subcomp, check_tzids, &tzdata);
-
-			if (!tzdata.found) {
-				status = GNOME_Evolution_Calendar_InvalidObject;
-				goto error;
-			}
-
-			if (!icalcomponent_get_uid (subcomp)) {
-				status = GNOME_Evolution_Calendar_InvalidObject;
-				goto error;
-			}
 		
-			comps = g_list_prepend (comps, subcomp);
-			break;
-		default:
-			/* Ignore it */
-			break;
-		}
+		if (child_kind != kind) {
+			/* remove the component from the toplevel VCALENDAR */
+			if (child_kind != ICAL_VTIMEZONE_COMPONENT)
+				del_comps = g_list_prepend (del_comps, subcomp);
 
+			subcomp = icalcomponent_get_next_component (toplevel_comp, ICAL_ANY_COMPONENT);
+			continue;
+		}
+			
+		tzdata.found = TRUE;
+		icalcomponent_foreach_tzid (subcomp, check_tzids, &tzdata);
+
+		if (!tzdata.found) {
+			status = GNOME_Evolution_Calendar_InvalidObject;
+			goto error;
+		}
+		
+		if (!icalcomponent_get_uid (subcomp)) {
+			status = GNOME_Evolution_Calendar_InvalidObject;
+			goto error;
+		}
+		
+		comps = g_list_prepend (comps, subcomp);
 		subcomp = icalcomponent_get_next_component (toplevel_comp, ICAL_ANY_COMPONENT);
 	}
 
 	/* Now we manipulate the components we care about */
 	for (l = comps; l; l = l->next) {
 		const char *uid, *rid;
-		char *calobj;
+		char *object, *old_object;
+		ECalComponent *old_comp;
 
 		subcomp = l->data;
 	
@@ -2082,9 +2136,6 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		e_cal_component_set_created (comp, &current);
 		e_cal_component_set_last_modified (comp, &current);
 
-		/* sanitize the component*/
-		sanitize_component (cbfile, comp); 
-
 		e_cal_component_get_uid (comp, &uid);
 		rid = e_cal_component_get_recurid_as_string (comp);
 
@@ -2092,31 +2143,28 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		case ICAL_METHOD_PUBLISH:
 		case ICAL_METHOD_REQUEST:
 		case ICAL_METHOD_REPLY:
-			if (e_cal_backend_file_get_object (backend, cal, uid, rid, &calobj)
-			    == GNOME_Evolution_Calendar_Success) {
-				char *returned_uid;
-
-				calobj = (char *) icalcomponent_as_ical_string (subcomp);
-				status = e_cal_backend_file_create_object (backend, cal, calobj, &returned_uid);
-				if (status != GNOME_Evolution_Calendar_Success)
-					goto error;
-
-				e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), calobj);
-			} else {
-				char *old_object;
-
-				calobj = (char *) icalcomponent_as_ical_string (subcomp);
-				status = e_cal_backend_file_modify_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object);
-				if (status != GNOME_Evolution_Calendar_Success)
-					goto error;
-
-				e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, calobj);
+			old_comp = lookup_component (cbfile, uid);
+			if (old_comp) {
+				old_object = e_cal_component_get_as_string (old_comp);
+				remove_component (cbfile, old_comp);
+				add_component (cbfile, comp, FALSE);
 
+				object = e_cal_component_get_as_string (comp);
+				e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, object);
+				g_free (object);
 				g_free (old_object);
+			} else {
+				add_component (cbfile, comp, FALSE);
+				
+				object = e_cal_component_get_as_string (comp);
+				e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), object);
+				g_free (object);
 			}
 			break;
 		case ICAL_METHOD_ADD:
 			/* FIXME This should be doable once all the recurid stuff is done */
+			status = GNOME_Evolution_Calendar_UnsupportedMethod;
+			goto error;
 			break;
 		case ICAL_METHOD_COUNTER:
 			status = GNOME_Evolution_Calendar_UnsupportedMethod;
@@ -2128,8 +2176,8 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 			break;
 		case ICAL_METHOD_CANCEL:
 			if (cancel_received_object (cbfile, subcomp)) {
-				calobj = (char *) icalcomponent_as_ical_string (subcomp);
-				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), icalcomponent_get_uid (subcomp), calobj);
+				object = (char *) icalcomponent_as_ical_string (subcomp);
+				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid, object);
 
 				/* remove the component from the toplevel VCALENDAR */
 				icalcomponent_remove_component (toplevel_comp, subcomp);
@@ -2144,6 +2192,16 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 
 	g_list_free (comps);
 	
+	/* Now we remove the components we don't care about */
+	for (l = del_comps; l; l = l->next) {
+		subcomp = l->data;
+		
+		icalcomponent_remove_component (toplevel_comp, subcomp);
+		icalcomponent_free (subcomp);		
+	}
+	
+	g_list_free (del_comps);
+
 	/* Merge the iCalendar components with our existing VCALENDAR,
 	   resolving any conflicting TZIDs. */
 	icalcomponent_merge_component (priv->icalcomp, toplevel_comp);
@@ -2157,10 +2215,12 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 }
 
 static ECalBackendSyncStatus
-e_cal_backend_file_send_objects (ECalBackendSync *backend, EDataCal *cal, const char *calobj)
+e_cal_backend_file_send_objects (ECalBackendSync *backend, EDataCal *cal, const char *calobj, GList **users,
+				 char **modified_calobj)
 {
-	/* FIXME Put in a util routine to send stuff via email */
-	
+	*users = NULL;
+	*modified_calobj = g_strdup (calobj);
+
 	return GNOME_Evolution_Calendar_Success;
 }
 
